<?php

namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");
use App\Model\RechargeApi;
use App\Model\Shopify\Product;
use App\Model\GetPackCount;
use App\CheckoutFunctions\checkoutFunctions;
use App\Helper\helper;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Log;


class AbandonedCheckoutController extends Controller
{

    private $modelObj;
    private $checkObj;
    private $packCount;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $helperObj;
    private $checkoutReturn;
    private $productObj;
    private $totalPrice = 0;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct() {

        $this->modelObj   = new RechargeApi();
        $this->packCount  = new GetPackCount();
        $this->productObj = new Product();
        $this->checkObj   = new checkoutFunctions();
        $this->helperObj  = new Helper();
    }

    /**
     * Create checkout
     */
    public function createAbandonedCheckout() {
        
        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing - token/email";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo json_encode($response);
            die();
        }

        $requestData             = $_REQUEST['data'];
        $parameterLineItemStatus = 0;
        $parameterShippingStatus = 0;
        $checkoutRequest         = array();
        $lineItems               = array();
        $shippingAddress         = array();

        $receivedData = json_decode($requestData);
        $receivedData = $this->helperObj->objectToArray($receivedData);

        if ((!isset($receivedData['email']) && empty($receivedData['email']))
        || (!isset($receivedData['email']) && empty($receivedData['checkoutToken']))) {

            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required parameters (email or token)",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $checkoutEmail = $receivedData['email'];
        $checkoutToken = $receivedData['checkoutToken'];

        $checkoutDbRecord = DB::table('user_checkouts')
        ->select('user_email','checkout_recharge_token','checkout_details')
        ->where('user_email', '=', $checkoutEmail )
        ->where('checkout_recharge_token', '=', $checkoutToken )
        ->get();
        
        if(!isset($checkoutDbRecord[0]->checkout_details)) {

            Log::channel('abandonedCheckoutLog')->info("createAbandonedCheckout: No checkout details 
            found in DB for email => ".$checkoutEmail." and token => ".$checkoutToken);

            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Invalid token or email.",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }
        
        
        $checkoutDetails = '';
        $externalCheckoutId = '';
        $externalCheckoutSource = '';
        $discountCode = '';
        
        if(isset($checkoutDbRecord[0]->checkout_details)){
            
            $checkoutDetails = $checkoutDbRecord[0]->checkout_details;
            $checkoutDetails = json_decode($checkoutDetails);
            
            if(isset($checkoutDetails->checkout->shipping_address)){
                $shippingAddress = $checkoutDetails->checkout->shipping_address;
            }
            
            if(isset($checkoutDetails->checkout->line_items)){
                $lineItems = $checkoutDetails->checkout->line_items;
            }
            
            if(isset($checkoutDetails->checkout->external_checkout_id)){
                $externalCheckoutId = $checkoutDetails->checkout->external_checkout_id;
            }
            
            if(isset($checkoutDetails->checkout->external_checkout_source)){
                $externalCheckoutSource = $checkoutDetails->checkout->external_checkout_source;
            }
            
            if(isset($checkoutDetails->checkout->discount_code)){
                $discountCode = $checkoutDetails->checkout->discount_code;
            }
            
        }
   
        $checkoutRequest = array(
            "checkout" => array(
                "line_items"               => $lineItems,
                "shipping_address"         => $shippingAddress,
                "email"                    => !empty($checkoutEmail) ? $checkoutEmail : '',
                "external_checkout_id"     => $externalCheckoutId,
                "external_checkout_source" => $externalCheckoutSource,
                "discount_code"            => $discountCode,

            )
        );
        
        $properties = $lineItems[0]->properties;
        $lineItemDetails = [];
        $firstName = '';
        $quizId = '';
        if(!empty($properties)){
            foreach($properties as $key => $value) {   
                if (strpos($key, 'Vitamin') !== false) {
                    $explodeValue = explode('|', $value);
                    $item = new \stdClass();
                    
                    if(!empty($explodeValue[3]) && !empty($explodeValue[2])){
                        $item->id = trim($explodeValue[3]);
                        $item->quantity = trim($explodeValue[2]);
                        $lineItemDetails[] = $item;
                    }
                   
                }
                if ($key == 'first_name') {
                    $firstName = $value;
                }
                if ($key == 'quiz_id') {
                    $quizId = $value;
                }
            }
        }
        
        
        foreach ($checkoutRequest['checkout'] as $key => $value) {
            if (empty($value)) {
                unset($checkoutRequest['checkout'][$key]);
            }
        }

        $checkoutReturn = $this->modelObj->createCheckout(json_encode($checkoutRequest));

        Log::channel('abandonedCheckoutLog')->info("createAbandonedCheckout: Checkout =>".
        json_encode($checkoutReturn));

        if (isset($checkoutReturn->error) || isset($checkoutReturn->errors) ||
            empty($checkoutReturn)) {

            $status = "failed";
            $error  = "Unknown error";
            if (isset($checkoutReturn->error)) {

                $error = json_encode($checkoutReturn->error);

            } else if (isset($checkoutReturn->errors)) {

                $error = json_encode($checkoutReturn->errors);
            }

            Log::channel('abandonedCheckoutLog')->info("createAbandonedCheckout: CheckoutError =>".$error);

            $response = array(
                "status"       => "failed",
                "message"      => "Failed to create the Checkout",
                "errorMessage" => $error,
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die;
        }

        $url = env('NURISH_CHECKOUT_URL', true) . $checkoutReturn->checkout->token .
            '?myshopify_domain=' . env('SHOPIFY_STORE', true);
        echo $_REQUEST['callback'] . "({'status':'success','url': '" . $url . "','errorMessage':'',
            'lineItems': '" . json_encode($lineItemDetails) . "','firstName': '" .$firstName. "',
            'quizId': '" .$quizId. "'})";
        die();
    }
    
    
    /**
     * Create checkout
     */
    public function createAbandonedCart() {
        
        if (!isset($_REQUEST['email']) || !isset($_REQUEST['email'])) {

            $message = "Endpoint parameters are missing - token/email";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo json_encode($response);
            die();
        }

        $parameterLineItemStatus = 0;
        $parameterShippingStatus = 0;
        $checkoutRequest         = array();
        $lineItems               = array();
        $shippingAddress         = array();

        $receivedData = $_REQUEST;

        if ((!isset($receivedData['email']) && empty($receivedData['email']))
        || (!isset($receivedData['email']) && empty($receivedData['checkoutToken']))) {

            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required parameters (email or token)",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $checkoutEmail = trim($receivedData['email']);
        $checkoutEmail = str_replace(' ','+',$checkoutEmail);
        $checkoutToken = trim($receivedData['checkoutToken']);
       

        $checkoutDbRecord = DB::table('user_checkouts')
        ->select('user_email','checkout_recharge_token','checkout_details')
        ->where('user_email', '=', $checkoutEmail )
        ->where('checkout_recharge_token', '=', $checkoutToken )
        ->get();
                
        if(!isset($checkoutDbRecord[0]->checkout_details)) {

            Log::channel('abandonedCheckoutLog')->info("createAbandonedCart: No checkout details 
            found in DB for email => ".$checkoutEmail." and token => ".$checkoutToken);

            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Invalid token or email.",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }
        
        $checkoutDetails = '';
        $externalCheckoutId = '';
        $externalCheckoutSource = '';
        $discountCode = '';
        
        if(isset($checkoutDbRecord[0]->checkout_details)){
            
            $checkoutDetails = $checkoutDbRecord[0]->checkout_details;
            $checkoutDetails = json_decode($checkoutDetails);
            
            if(isset($checkoutDetails->checkout->shipping_address)){
                $shippingAddress = $checkoutDetails->checkout->shipping_address;
            }
            
            if(isset($checkoutDetails->checkout->line_items)){
                $lineItems = $checkoutDetails->checkout->line_items;
            }
            
            if(isset($checkoutDetails->checkout->external_checkout_id)){
                $externalCheckoutId = $checkoutDetails->checkout->external_checkout_id;
            }
            
            if(isset($checkoutDetails->checkout->external_checkout_source)){
                $externalCheckoutSource = $checkoutDetails->checkout->external_checkout_source;
            }
            
            if(isset($checkoutDetails->checkout->discount_code)){
                $discountCode = $checkoutDetails->checkout->discount_code;
            }
            
        }
    
        $checkoutRequest = array(
            "checkout" => array(
                "line_items"               => $lineItems,
                "shipping_address"         => $shippingAddress,
                "email"                    => !empty($checkoutEmail) ? $checkoutEmail : '',
                "external_checkout_id"     => $externalCheckoutId,
                "external_checkout_source" => $externalCheckoutSource,
                "discount_code"            => $discountCode,

            )
        );
        
        $properties = $lineItems[0]->properties;
        $lineItemDetails = [];
        if(!empty($properties)){
            foreach($properties as $key => $value) {   
                if (strpos($key, 'Vitamin') !== false) {
                    $explodeValue = explode('|', $value);
                    $item = new \stdClass();
                    
                    if(!empty($explodeValue[3]) && !empty($explodeValue[2])){
                        $item->id = trim($explodeValue[3]);
                        $item->quantity = trim($explodeValue[2]);
                        $lineItemDetails[] = $item;
                    }
                   
                }
            }
        }
        
        
        foreach ($checkoutRequest['checkout'] as $key => $value) {
            if (empty($value)) {
                unset($checkoutRequest['checkout'][$key]);
            }
        }

        $checkoutReturn = $this->modelObj->createCheckout(json_encode($checkoutRequest));

        Log::channel('abandonedCheckoutLog')->info("createAbandonedCart: Checkout =>". 
        json_encode($checkoutReturn));

        if (isset($checkoutReturn->error) || isset($checkoutReturn->errors) ||
            empty($checkoutReturn)) {

            $status = "failed";
            $error  = "Unknown error";
            if (isset($checkoutReturn->error)) {

                $error = json_encode($checkoutReturn->error);

            } else if (isset($checkoutReturn->errors)) {

                $error = json_encode($checkoutReturn->errors);
            }

            Log::channel('abandonedCheckoutLog')->info("createAbandonedCart: CheckoutError =>". $error);

            $response = array(
                "status"       => "failed",
                "message"      => "Failed to create the Checkout",
                "errorMessage" => $error,
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die;
        }

        $url = env('NURISH_CHECKOUT_URL', true) . $checkoutReturn->checkout->token .
            '?myshopify_domain=' . env('SHOPIFY_STORE', true);
            
        return redirect()->to($url);
        //exit();
        
        echo $_REQUEST['callback'] . "({'status':'success','url': '" . $url . "','errorMessage':'',
            'lineItems': '" . json_encode($lineItemDetails) . "'})";
        die();
    }

}
