<?php

namespace App\Http\Controllers;
header("Access-Control-Allow-Origin: *");
use App\Model\IterableApi;
use App\CheckoutFunctions\checkoutFunctions;
use App\Helper\helper;
use Illuminate\Http\Request;
use Log;

class IterableController extends Controller
{

    private $modelObj;
    private $checkObj;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $helperObj;
    private $checkoutReturn;
    private $totalPrice = 0;
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->modelObj = new IterableApi();
        $this->checkObj = new checkoutFunctions();
        $this->helperObj = new Helper();
    }

    /**
     * Get user Iterable details
     */
    public function getUserIterableStatus() {

        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);

        if(empty($receivedData->email)) {

            $response = array(
                "status" => "Failed",
                "errorMessage" => "Missing required parameters or the request is not correct",
            );

            echo $_REQUEST['callback'].'('.json_encode($response).')';
            die();
         }

         $subscriptionResponse = $this->modelObj->getIterableSubscription($receivedData->email);


         if (!empty($subscriptionResponse->msg) || empty($subscriptionResponse)) {

            $status = "failed";
            $error = "Unknown error";
            if (isset($subscriptionResponse->msg)) {

                $error = "Invalid Email";

            } else if (isset($subscriptionResponse->errors)) {

                $error = "Invalid Email";
            }

            Log::channel('iterableLog')->info("getUserIterableStatus: Failed to get the Iterable details for Email (".
            $receivedData->email."), Error: " .$error);

            $response = array(
                "status" => "failed",
                "message" => "Failed to get the Iterable details",
                "errorMessage" => $error,
            );

            echo $_REQUEST['callback'].'('.json_encode($response).')';
            die();
        }

        $responseData = array(
            'unsubscribedChannelIds' => $subscriptionResponse->user->dataFields->unsubscribedChannelIds,
            'subscribedMessageTypeIds' => $subscriptionResponse->user->dataFields->subscribedMessageTypeIds,
            'unsubscribedMessageTypeIds' => $subscriptionResponse->user->dataFields->unsubscribedMessageTypeIds

        );

        $response = array(
            "status" => "success",
            "response" => $responseData,
            "errorMessage" => "",
        );

        echo $_REQUEST['callback'].'('.json_encode($response).')';
        die();

    }

    /**
     * Update user Iterable details
     */
     public function updateIterableSubscription() {

        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);

        if(empty($receivedData) || empty($receivedData->email)) {

            $response = array(
                "status" => "Failed",
                "errorMessage" => "Missing required parameters or the request is not correct",
            );

            echo $_REQUEST['callback'].'('.json_encode($response).')';
            die();
         }


         $updateRequest = array(
            "email" => $receivedData->email,
            "unsubscribedChannelIds" => 
            array_map('intval', $receivedData->unSubscribeChannelIds),
            "unsubscribedMessageTypeIds" => 
            array_map('intval', $receivedData->unSubscribePreferenceIds),
            "subscribedMessageTypeIds" => 
            array_map('intval', $receivedData->subscribePreferenceIds)
        );

        $subscriptionResponse = $this->modelObj->updateUserIterableSubscription(json_encode($updateRequest));

        if (!empty($subscriptionResponse->msg) || empty($subscriptionResponse)) {

            $status = "failed";
            $error = "Unknown error";
            if (isset($subscriptionResponse->msg)) {

                $error = "Invalid parameters";

            } else if (isset($subscriptionResponse->errors)) {

                $error = "Invalid parameters";
            }

            Log::channel('iterableLog')->info("updateIterableSubscription: Failed to update the Iterable subscription 
            details for Email (".$receivedData->email."), Error: " .$error);

            $response = array(
                "status" => "failed",
                "message" => "Failed to update the Iterable subscription",
                "errorMessage" => $error,
            );

            echo $_REQUEST['callback'].'('.json_encode($response).')';
            die();
        }

        $response = array(
            "status" => "success",
            "response" => 'updated',
            "errorMessage" => "",
        );

        echo $_REQUEST['callback'].'('.json_encode($response).')';
        die();

     }

    /**
      * add user to Iterable List
    */
    public function addUserToIterableList(){

        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);

        $subscriptionRequest = array(

            "listId" => intval(env('ITERABLE_SMS_SUBSCRIBE_LIST_ID', true)),
            "subscribers" => array(
                array(
                    "email" => $receivedData->email,
                    "dataFields" => array(
                    "phoneNumber" => $receivedData->mobileNumber,
                    "First Name" => $receivedData->firstName,
                    "Last Name" => $receivedData->lastName,
                    "default_address" => array(
                        "zip" => $receivedData->postalCode
                    )
                  )
               )
            )

        );

        $createUserSubscribeListResponse = 
        $this->modelObj->addSubscriberToIterableList(json_encode($subscriptionRequest));

        if (!empty($createUserSubscribeListResponse->msg) || 
        empty($createUserSubscribeListResponse)) {

            $status = "failed";
            $error = "Unknown error";
            if (isset($subscriptionResponse->msg)) {

                $error = "Invalid parameters";

            } else if (isset($subscriptionResponse->errors)) {

                $error = "Invalid parameters";
            }

            Log::channel('iterableLog')->info("addUserToIterableList: Failed to update the 
            Iterable subscription details for Email (".$receivedData->email."), Error: " .$error);

            $response = array(
                "status" => "failed",
                "message" => "Failed to update the Iterable subscription",
                "errorMessage" => $error,
            );

            echo $_REQUEST['callback'].'('.json_encode($response).')';
            die();
        }

        $response = array(
            "status" => "success",
            "response" => 'updated',
            "errorMessage" => "",
        );

        echo $_REQUEST['callback'].'('.json_encode($response).')';
        die();

      }

}
