<?php

namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");
use App\Model\RechargeApi;
use App\Model\Shopify\Product;
use App\Model\GetPackCount;
use App\CheckoutFunctions\checkoutFunctions;
use App\Helper\helper;
use Illuminate\Http\Request;
use Log;

class RechargeAddressController extends Controller
{

    private $modelObj;
    private $checkObj;
    private $packCount;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $helperObj;
    private $checkoutReturn;
    private $productObj;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct() {

        $this->modelObj   = new RechargeApi();
        $this->packCount  = new GetPackCount();
        $this->productObj = new Product();
        $this->checkObj   = new checkoutFunctions();
        $this->helperObj  = new Helper();
    }

    /**
     * Update subscription shipping rate when updating the address
     */
    public function updateSubscriptionByAddressID() {

        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo json_encode($response);
            die();

        }
        $requestData = $_REQUEST['data'];
        $receivedData = json_decode($requestData);

        if(empty($receivedData->customer_id) || empty($receivedData->address_id)) {

            $response = array(
                "status"       => "Failed",
                "errorMessage" => "The customerID or addressID is empty",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        } 

        $customerSubscriptions = $this->modelObj->getAllSubscriptionsByCustomerID($receivedData->customer_id);

        if (!empty($customerSubscriptions->error) || !empty($customerSubscriptions->errors)
        || empty($customerSubscriptions->subscriptions)) {
    
            $status = "failed";
            $error  = "Unknown error";
            if (!empty($customerSubscriptions->error)) {

                $error = $customerSubscriptions->error;

            } else if (!empty($customerSubscriptions->errors)) {

                $error = json_encode($customerSubscriptions->errors);

            } else if (empty($customerSubscriptions->subscriptions)) {

                $error = "Failed to get the subscriptions";
            }

            Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: Failed to get the subscription details, 
            error: " .$error.", Endpoint Request:".json_encode($receivedData));

            $response = array(
            "status"       => "Failed",
            "errorMessage" => "Failed to get the subscription details",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        if (empty($customerSubscriptions) || !isset($customerSubscriptions)) {

            $response = array(
                "status"       => "Failed",
                "message"      => "There is no subscription in the given customer id",
                "errorMessage" => "",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $checkOverRideFreeShippingStatus = 0;
        $addressIdFoundStatus = 0;
        $subscriptionPrice = 0;
        $shippingRateUpdateStatus = 0;
        $freeShippingThreshold = 0;

        foreach ($customerSubscriptions->subscriptions as $subValue) {

            if (trim($subValue->address_id) == trim($receivedData->address_id)) {

                $addressIdFoundStatus = 1;
                $subscriptionPrice = trim($subValue->price);
                if (!empty($subValue->properties)) {

                    foreach ($subValue->properties as $propKey => $propValue) {
    
                        $subscriptionPropertyCheck = strtolower(trim($propValue->name));
                        if ($subscriptionPropertyCheck == 'override_free_shipping') {
    
                            $shippingOverrideValue = !empty($propValue->value) ? $propValue->value : false;
                            if ($shippingOverrideValue && $shippingOverrideValue == "true") {
                                
                                $checkOverRideFreeShippingStatus = 1;
                            }
                        }
                        if ($subscriptionPropertyCheck == 'free_shipping_threshold') {
 
                            $shippingThresholdValue = !empty($propValue->value) ? trim($propValue->value) : 
                            number_format((float)0, 2, '.', '');
                            if (!empty($shippingThresholdValue)) {
                                
                                $freeShippingThreshold = $shippingThresholdValue;
                                $freeShippingThreshold = round($freeShippingThreshold);
                                $freeShippingThreshold = number_format((float)$freeShippingThreshold, 2, '.', '');
                            }
                        }
                    }
                }
            }
        }

        if ($addressIdFoundStatus == 0) {

            $response = array(
                "status"       => "Failed",
                "message"      => "The address Id that passed in the request is not found in the subscription",
                "errorMessage" => "",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        // If override_free_shipping is not found in the subscription LIP
        if ($checkOverRideFreeShippingStatus == 0) {

            $usShippingRateAdjustableCountries = array(

                "alabama","arizona","arkansas","california","colorado","connecticut","delaware","florida","georgia",
                "idaho","illinois","indiana","iowa","kansas","kentucky","louisiana","maine","maryland","massachusetts",
                "michigan","minnesota","mississippi","missouri","montana","nebraska","nevada","new-hampshire",
                "new-jersey","new-mexico","new-york","north-carolina","north-dakota","ohio","oklahoma",
                "oregon","pennsylvania","rhode-island","south-carolina","south-dakota","tennessee","texas",
                "utah","vermont","virginia","washington","west-virginia","wisconsin","wyoming"
            );

            $usShippingRateNonAdjustableCountries = array(
                "alaska","hawaii"
            );

            $subscriptionShippingZoneIds = [96075317330,96075284562,94991646791,94991679559];

            $shopifyDomesticShippingPrice = '';
            $shopifyNonDomesticShippingPrice = '';

            $shippingZones = $this->productObj->getShopifyShippingZone();
            if (!empty($shippingZones['error']) || !empty($shippingZones['errors']) ||
                empty($shippingZones)) {

                $status = "failed";
                $error  = "Unknown error";
                if (!empty($shippingZones['error'])) {

                    $error = $shippingZones['error'];

                } else if (!empty($shippingZones['errors'])) {

                    $error = json_encode($shippingZones['errors']);

                } else if (empty($shippingZones)) {

                    $error = "Failed to get the Shopify shipping zone details";
                }

                Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: Failed to get the Shopify shipping zone 
                details, error: " .$error.", Endpoint Request:".json_encode($receivedData));

                $response = array(
                    "status"       => "Failed",
                    "errorMessage" => "Failed to get the Shopify shipping zone details",
                );

                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                die();
            }

            if (isset($shippingZones['shipping_zones'])) {

                foreach ($shippingZones['shipping_zones'] as $shipValue) {

                    $shippingZoneId = !empty($shipValue['id']) ? trim($shipValue['id']) : '';
                    $shippingZoneName = strtolower(trim($shipValue['name']));
                    if (strpos($shipValue['name'], ",") !== false ) {

                        $shippingZoneName = str_replace(" ", "", $shippingZoneName);
                        $shippingZoneName = str_replace(",", "-", $shippingZoneName);
                    }

                    if (($shippingZoneName == 'domestic' || $shippingZoneName == 'alaska-hawaii') && 
                    in_array($shippingZoneId, $subscriptionShippingZoneIds)) {

                        if(!empty($shipValue['price_based_shipping_rates'])) {

                            foreach ($shipValue['price_based_shipping_rates'] as $priceValue) {

                                $shippingName = strtolower(trim($priceValue['name']));
                                $shippingName = str_replace(" ", "-", $shippingName);

                                if($shippingName == 'standard-shipping') {

                                    if ($shippingZoneName == 'domestic') {

                                        $shopifyDomesticShippingPrice = $priceValue['price'];

                                    } else {

                                        $shopifyNonDomesticShippingPrice = $priceValue['price'];

                                    }
                                }
                            }
                        }
                    }
                }
            }

            $userAddressDetails = $this->modelObj->getAddressByAddressID($receivedData->address_id);

            if (!empty($userAddressDetails->error) || !empty($userAddressDetails->errors) ||
            empty($userAddressDetails)) {

                $status = "failed";
                $error  = "Unknown error";
                if (!empty($userAddressDetails->error)) {

                    $error = $userAddressDetails->error;

                } else if (!empty($userAddressDetails->errors)) {

                    $error = json_encode($userAddressDetails->errors);

                } else if (empty($userAddressDetails)) {

                    $error = "Failed to get the address details by address ID";
                }

                Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: Failed => " .$error.", 
                AddressId:".$receivedData->address_id);

                $response = array(
                    "status"       => "Failed",
                    "errorMessage" => "Failed to get the address details to override the shipping details",
                );
                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                die();
            }

            $addressProvince = '';
            if (!empty($userAddressDetails->address->province)) {

                $addressProvince = strtolower(trim($userAddressDetails->address->province));
                $addressProvince = str_replace(" ", "-", $addressProvince);
    
            } else {
    
                $addressProvince = '';
            }

            $shippingUpdateStatus = 0;
            $shippingPrice = '';
            $shippingCode = '';
            $freeShippingThreshold = !empty($freeShippingThreshold) ? $freeShippingThreshold : 
            number_format((float)20, 2, '.', '');

            if (!empty($addressProvince) && in_array($addressProvince, $usShippingRateAdjustableCountries)) {

                if ($subscriptionPrice >= $freeShippingThreshold) {
                    $shippingPrice = 0;
                    $shippingCode = "Free Shipping";
                } else {
                    $shippingPrice = !empty($shopifyDomesticShippingPrice) ? $shopifyDomesticShippingPrice : 8;
                    $shippingCode = "Standard Shipping";
                }

                $shippingUpdateStatus = 1;

            } elseif (!empty($addressProvince) && in_array($addressProvince, $usShippingRateNonAdjustableCountries)) {

                $shippingPrice = !empty($shopifyNonDomesticShippingPrice) ? $shopifyNonDomesticShippingPrice : 8;
                $shippingCode = "Standard Shipping";

                $shippingUpdateStatus = 1;

            } else {

                if (!empty($freeShippingThreshold)) {

                    if ($subscriptionPrice >= $freeShippingThreshold) {
                        $shippingPrice = 0;
                        $shippingCode = "Free Shipping";
                    }
                } else {

                    $shippingPrice = !empty($shopifyDomesticShippingPrice) ? $shopifyDomesticShippingPrice : 8;
                    $shippingCode = "Standard Shipping";
                }

                $shippingUpdateStatus = 1;
            }

            if ($shippingUpdateStatus == 1) {

                $shippingUpdateRequest = array(

                    "shipping_lines_override" => array(
                    array(
                        "price" => $shippingPrice,
                        "code"  => $shippingCode,
                        "title" => "Standard Shipping"

                    )
                )
                );

                $updatedShippingRateResponse = $this->modelObj->updateShippingRateByAddress(
                    $receivedData->address_id, json_encode($shippingUpdateRequest));

                    if (isset($updatedShippingRateResponse->error) || isset($updatedShippingRateResponse->errors) ||
                    empty($updatedShippingRateResponse)) {
        
                    $status = "failed";
                    $error  = "Unknown error";
                    if (isset($updatedShippingRateResponse->error)) {
        
                        $error = $updatedShippingRateResponse->error;
        
                    } else if (isset($updatedShippingRateResponse->errors)) {
        
                        $error = json_encode($updatedShippingRateResponse->errors);
        
                    } else if (empty($updatedShippingRateResponse)) {

                        $error = "Failed to update the Shipping amount";
                    }

                    Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: Failed to update the Shipping amount, 
                    error: " .$error.", AddressId:".$receivedData->address_id);

                    $response = array(
                        "status"       => "Failed",
                        "errorMessage" => "Failed to update the shipping amount",
                    );
                    echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                    die();
                }
            }

            $shippingRateUpdateStatus = 1;

        } else if ($checkOverRideFreeShippingStatus == 1) {

            $existingShippingPrice = '';
            $shippingPrice = '';
            $freeShippingThreshold = 0;
            $userAddressDetails = $this->modelObj->getAddressByAddressID($receivedData->address_id);
            if (!empty($userAddressDetails->error) || !empty($userAddressDetails->errors) ||
                empty($userAddressDetails)) {

                $status = "failed";
                $error  = "Unknown error";
                if (!empty($userAddressDetails->error)) {

                    $error = $userAddressDetails->error;

                } else if (!empty($userAddressDetails->errors)) {

                    $error = json_encode($userAddressDetails->errors);

                } else if (empty($userAddressDetails)) {

                    $error = "Failed to get the address details by address ID";
                }

                Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: Failed => " .$error.", 
                AddressId:".$receivedData->address_id);

                $response = array(
                    "status"       => "Failed",
                    "errorMessage" => "Failed to get the address details to override the shipping details",
                );
                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                die();
            }

            $addressOriginalShipping = !empty($userAddressDetails->address->original_shipping_lines[0]->price) ?
            (float)$userAddressDetails->address->original_shipping_lines[0]->price : 0;
            $addressOverRideFreeShipping = !empty($userAddressDetails->address->shipping_lines_override[0]->price) ? 
            (float)$userAddressDetails->address->shipping_lines_override[0]->price : 0;
            $existingShippingPrice = !empty($addressOverRideFreeShipping) ? $addressOverRideFreeShipping : 
            $addressOriginalShipping;

            if ($addressOverRideFreeShipping > 0 || $addressOriginalShipping > 0) {

                $shippingUpdateRequest = array(

                    "shipping_lines_override" => array(
                        array(
                            "code" => "Free shipping",
                            "price"=> "0.00",
                            "title"=> "Free shipping"
                        )
                    )
                );

                $updatedShippingRateResponse = $this->modelObj->updateShippingRateByAddress(
                    $receivedData->address_id, json_encode($shippingUpdateRequest));

                if (isset($updatedShippingRateResponse->error) || isset($updatedShippingRateResponse->errors) ||
                empty($updatedShippingRateResponse)) {

                    $status = "failed";
                    $error  = "Unknown error";
                    if (isset($updatedShippingRateResponse->error)) {

                        $error = $updatedShippingRateResponse->error;

                    } else if (isset($updatedShippingRateResponse->errors)) {

                        $error = json_encode($updatedShippingRateResponse->errors);

                    } else if (empty($updatedShippingRateResponse)) {

                        $error = "Failed to update the Shipping amount";
                    }

                    Log::channel('rechargeAddressLog')->info("updateSubscriptionByAddressID: Failed => " .$error.", 
                    AddressId:".$receivedData->address_id);

                    $response = array(
                        "status"       => "Failed",
                        "errorMessage" => "Failed to update the shipping amount",
                    );
                    echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                    die();
                }
                $shippingRateUpdateStatus = 1;
            }
        }

        if($shippingRateUpdateStatus == 0) {

            $response         = array(
                "status"             => "success",
                "message"            => "There is no subscriptions in the given addressID",
                "errorMessage"       => "",
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        if ($shippingRateUpdateStatus == 1) {

            $response         = array(
                "status"             => "success",
                "message"            => "Successfully updated the subscriptions shipping amount of the given user",
                "errorMessage"       => "",
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }
    }
}
