<?php
namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\Model\RechargeApi;
use App\Model\IterableApi;
use App\Helper\Helper;
use Log;

class IterableCheckoutWebhook implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $helperObj;
    private $iterableObj;
    private $rechargeObj;
    private $customerID;
    private $properties;
    private $requestData;
    //private $logFile;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($requestData)
    {
        $this->requestData = $requestData;
        $this->customerID  = $this->requestData->order->customer_id;
        //$this->logFile = $logPath;
        $this->iterableObj = new IterableApi();
        $this->rechargeObj = new RechargeApi();
        $this->helperObj = new Helper();
    }

    public function handle()
    {

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($this->customerID);
        Log::channel('iterableWebhookLog')->info("IterableCheckoutWebhook: Reached to onetime create job");

        if (isset($customerDetails->error) || isset($customerDetails->errors) || empty($customerDetails->customer->email) ||
        empty($customerDetails)) {

            $status = "failed";
            $error = "Unknown error";
            if (isset($customerDetails->error)) {

                $error = $customerDetails->error;

            } else if (isset($customerDetails->errors)) {

                $error = "Request is not correct or missing the required parameters";

            }

            Log::channel('iterableWebhookLog')->info("IterableCheckoutWebhook: Created subscription 
            iterable event - Failed to get the Recharge user details for CustomerID (".
            $this->customerID."), Error: " . $error);

            die();
        }

        

        $eventRequest = array(
            "events" => array(
            array("email" => $customerDetails->customer->email,
            "eventName" => "OneTimePurchase",
                "dataFields" => array(
                    "userName" => $customerDetails->customer->first_name.' '.$customerDetails->customer->last_name),
                    "rechargeHash" => $customerDetails->customer->hash,
             )

            )
        );
        

        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));

        if(isset($eventResponse->successCount) || $eventResponse->successCount > 0) {

            Log::channel('iterableWebhookLog')->info("IterableCheckoutWebhook: Created OTP iterable event - 
            Successfully created event for Email (".strtolower($customerDetails->customer->email).")");
        } else {
            Log::channel('iterableWebhookLog')->info("IterableCheckoutWebhook: Created OTP iterable event - 
            Failed to create event for Email (".strtolower($customerDetails->customer->email).")");
        }
        
        $userIterableUpdateRequest = array(
            "email" => $customerDetails->customer->email,
            "dataFields" => array(
                "rechargeHash" => $customerDetails->customer->hash,
                "rechargeOrderId" => $this->requestData->order->id
            )
        );

        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));

        if(strtolower(trim($userIterableUpdateResponse->code)) == 'success') {

            Log::channel('iterableWebhookLog')->info("IterableCheckoutWebhook: User rechargeHash  - Successfully
            updated the user recharge Hash in Iterable for Email (".strtolower($customerDetails->customer->email).")");
        } else {
            Log::channel('iterableWebhookLog')->info("IterableCheckoutWebhook: User rechargeHash - Failed to update
            the user recharge Hash in Iterable for Email (".strtolower($customerDetails->customer->email).")");
        }
        

    }

}
