<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\Model\RechargeApi;
use App\Model\IterableApi;
use App\Helper\Helper;
use Log;

class IterableSubscriptionChargeUpcomingEvent implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $helperObj;
    private $iterableObj;
    private $rechargeObj;
    private $webhookResponse;
    //private $logFile;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($webhookResponse)
    {
        $this->webhookResponse = $webhookResponse;
        //$this->logFile = $logPath;
        $this->iterableObj = new IterableApi();
        $this->rechargeObj = new RechargeApi();
        $this->helperObj = new Helper();
    }

    public function handle()
    {
        $nextChargeDate = '';
        $subscriptionLineItemsArray = [];
        $subscriptionLineItems = '';
        $giftSubscriptionStatus = 0;
        if(!empty($this->webhookResponse['charge']['line_items'])) {

            $nextChargeDate = !empty($this->webhookResponse['charge']['scheduled_at']) ?
            $this->webhookResponse['charge']['scheduled_at'] : '';

            foreach ($this->webhookResponse['charge']['line_items'] as $chargeValue) {

                if(!empty($chargeValue['properties'])) {

                    foreach ($chargeValue['properties'] as $property) {

                        $propertyName = strtolower(trim($property['name']));
                        if ($propertyName == 'giftee_email') {

                            $propertyValueForGiftSubChecking = !empty($property['value']) ? trim($property['value']) : '';
                            if (!empty($propertyValueForGiftSubChecking)) {

                                $giftSubscriptionStatus = 1;
                            }
                        }
                        if (strpos($propertyName, 'vitamin') !== false) {

                            $propertyValue = trim($property['value']);

                            if(strpos($propertyValue, '|') !== false) {

                                $explodePropertyValue = explode('|', $propertyValue);

                                if(count($explodePropertyValue) == 5) {

                                    $subscriptionLineItemsArray[] = trim($explodePropertyValue[0]);

                                }
                            }
                        }
                    }
                }
            }
        }
        
        // Skipping if it is a gift subscription
        if ($giftSubscriptionStatus == 1) {

            $errorMessage = "Upcoming Charge iterable event: Skipped - The subscription is a gift subscription
            (Email: ".$this->webhookResponse['charge']['email'].", SubscriptionId: ".
            $this->webhookResponse['charge']['line_items'][0]['subscription_id'].")";
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: ".$errorMessage);
            die();
        }

        $subscriptionId = !empty($this->webhookResponse['charge']['line_items'][0]['subscription_id']) ? 
        trim($this->webhookResponse['charge']['line_items'][0]['subscription_id']) : "";
        
        // $firstName = '';
        // if(isset($this->webhookResponse['charge']['first_name']) && !empty($this->webhookResponse['charge']['first_name'])) {
        //     $firstName = $this->webhookResponse['charge']['first_name'];
        // }
        
        // $discountCode = '';
        // if(isset($this->webhookResponse['charge']['discount_codes']) && !empty($this->webhookResponse['charge']['discount_codes'])) {
            
        //     $discount = $this->webhookResponse['charge']['discount_codes'][0];
        //     $discountCode = $discount['code'];
            
        // }
        
        // $totalPrice = '';
        // if(isset($this->webhookResponse['charge']['total_price']) && !empty($this->webhookResponse['charge']['total_price'])) {
        //     $totalPrice = $this->webhookResponse['charge']['total_price'];
        // }
        
        

        if (!isset($subscriptionLineItemsArray) || empty($subscriptionLineItemsArray) ||
        empty($this->webhookResponse['charge']['email'])) {

            if(empty($this->webhookResponse['charge']['email'])) {

                $error = "Upcoming Charge iterable event - The email address is not valid or the field is empty";
            } else {

                $error = "Upcoming Charge iterable event - The charge line items are not valid or the line
                items are empty(Email - ".$this->webhookResponse['charge']['email'].")";
            }
            //$status = "failed";
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: ".$error);
            die();
        }

        if (!empty($nextChargeDate)) {

            $nextChargeDate = strtotime($nextChargeDate);
            $nextChargeDate = date("Y-m-d H:i:s", $nextChargeDate);
        }
        $subscriptionLineItems = implode(' | ', $subscriptionLineItemsArray);

        $rechargeCustomerHash = !empty($this->webhookResponse['charge']['customer_hash']) ? 
        trim($this->webhookResponse['charge']['customer_hash']) : '';
        $customerID = $this->webhookResponse['charge']['customer_id'];
        $customerSubscriptionDetails = $this->rechargeObj->getAllActiveSubscriptionsByCustomerID($customerID);

        $contactPropertyNextChargeDate = $nextChargeDate;
        $contactPropertySubscriptionItems = $subscriptionLineItems;

        if (!isset($customerSubscriptionDetails->subscriptions)) {
            if (isset($customerSubscriptionDetails->error) || isset($customerSubscriptionDetails->errors)) {
                if (isset($customerSubscriptionDetails->error)) {
                    $error = json_encode($customerSubscriptionDetails->error);
                } else if (isset($customerSubscriptionDetails->errors)) {
                    $error = json_encode($customerSubscriptionDetails->errors);
                }
                Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Failed to fetch subscription 
                details from recharge, Error is: " . $error);
            } else {
                Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Failed to fetch subscription 
                details from recharge, Error details not available");
            }
        } else {
            if (count($customerSubscriptionDetails->subscriptions) > 1) {
                $contactPropertyNextChargeDate = NULL;
                $contactPropertySubscriptionItems = "Multiple Subscriptions";
            }
        }

        // Check duplicate upcoming events
        $eventCreationStatus = '';
        $eventCreationStatus = $this->checkUpcomingEventDuplicateStatus();

        if ($eventCreationStatus) {

            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Upcoming Charge event is 
            already created in iterable, subscriptionId: ".$subscriptionId);
            exit();
        }

        $eventRequest = array(
            "events" => array(
            array("email" => strtolower($this->webhookResponse['charge']['email']),
            "eventName" => "Upcoming Charge",
            "dataFields" => array(
                "nextChargeDate" => $nextChargeDate,
                "subscriptionItems" => $subscriptionLineItems,
                // "totalPrice" => $totalPrice,
                // "firstName" => $firstName,
                // "couponCode" => $discountCode,
                "rechargeHash" => $rechargeCustomerHash,
                "subscriptionId" => $subscriptionId
                )
              )
            )
        );

        $userIterableUpdateRequest = array(
            "email" => strtolower($this->webhookResponse['charge']['email']),
            "dataFields" => array(
                "nextChargeDate" => $contactPropertyNextChargeDate,
                "subscriptionItems" => $contactPropertySubscriptionItems
            )
        );

        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));

        if(strtolower(trim($userIterableUpdateResponse->code)) == 'success') {

            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Charge/Upcoming user 
            contact update - Successfully updated the charge nextChargeDate and subscriptionItems in Iterable user 
            contact details(Email - ".strtolower($this->webhookResponse['charge']['email']).")");
        } else {
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Charge/Upcoming user 
            contact update - Failed to update the charge nextChargeDate and subscriptionItems in Iterable user contact 
            details (Email  - ".strtolower($this->webhookResponse['charge']['email']).")");
        }

        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));

        if(isset($eventResponse->successCount) || $eventResponse->successCount > 0) {

            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Upcoming Charge iterable 
            event - Successfully created event for Email (".strtolower($this->webhookResponse['charge']['email']).")");
        } else {
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Upcoming Charge iterable event - 
            Failed to create event for Email (".strtolower($this->webhookResponse['charge']['email']).")");
        }
    }

    /**
     * Check upcoming charge duplicate status
     */
    public function checkUpcomingEventDuplicateStatus() {

        $eventCreateStatus = "";
        $customerID = !empty($this->webhookResponse['charge']['customer_id']) ? 
        trim($this->webhookResponse['charge']['customer_id']) : "";
        $eventName = "Upcoming Charge";
        $eventName = strtolower(preg_replace('/\s+/', '', $eventName));
        $subscriptionId = !empty($this->webhookResponse['charge']['line_items'][0]['subscription_id']) ? 
        trim($this->webhookResponse['charge']['line_items'][0]['subscription_id']) : "";
        $currentDate = date('Y-m-d');
        $subscriptionChargeDate = !empty($this->webhookResponse['charge']['scheduled_at']) ? 
        trim($this->webhookResponse['charge']['scheduled_at']) : "";

        if (empty($customerID)) {

            $eventCreateStatus = false;
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Not found the customerId in 
            the webhook response, Request:".json_encode($this->webhookResponse));
            return $eventCreateStatus;
        }

        // Get customer details
        sleep(1);
        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($customerID);
        if (!empty($customerDetails->error) || !empty($customerDetails->errors) || empty($customerDetails->customer)) {

            $customerGetError  = "Unknown error";
            if (!empty($customerDetails->error)) {

                $customerGetError = json_encode($customerDetails->error);
            } else if (!empty($customerDetails->errors)) {

                $customerGetError = json_encode($customerDetails->errors);
            } else if (empty($customerDetails->customer)) {

                $customerGetError = 'Failed to get the customer details from recharge';
            }

            $eventCreateStatus = false;
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Failed to get the 
            customer details from recharge, customerId: ".$customerID.", Error:".$customerGetError);
            return $eventCreateStatus;
        }

        $shopifyCustomerId = !empty($customerDetails->customer->shopify_customer_id) ? 
        trim($customerDetails->customer->shopify_customer_id) : "";

        if (empty($shopifyCustomerId)) {

            $eventCreateStatus = false;
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Not found the customerId, 
            Customer details from recharge:".json_encode($customerDetails));
            return $eventCreateStatus;
        }

        $limit = 100;
        $eventResponse = $this->iterableObj->getIterableEventsByShopifyCustomerId($limit, $shopifyCustomerId);

        if (empty($eventResponse->events) || !empty($eventResponse->error) || !empty($eventResponse->errors)) {

            $iterableError  = "Unknown error";
            if (!empty($eventResponse->error)) {

                $iterableError = json_encode($eventResponse->error);
            } else if (!empty($eventResponse->errors)) {

                $iterableError = json_encode($eventResponse->errors);
            } else if (empty($eventResponse->customer)) {

                $iterableError = 'Failed to get the event details from iterable';
            }

            $eventCreateStatus = false;
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Failed to get the 
            event details from iterable, shCustomerId: ".$shopifyCustomerId.", Error:".$iterableError);
            return $eventCreateStatus;
        }

        foreach ($eventResponse->events as $eventKey => $eventValue) {

            $iterableEventName = !empty($eventValue->eventName) ? trim($eventValue->eventName) : "";
            $iterableSubscriptionId = !empty($eventValue->dataFields->subscriptionId) ? 
            trim($eventValue->dataFields->subscriptionId) : "";
            $iterableNextChargeDate = !empty($eventValue->dataFields->nextChargeDate) ? 
            trim($eventValue->dataFields->nextChargeDate) : "";
            if (!empty($iterableNextChargeDate)) {
                $iterableNextChargeDate = date('Y-m-d', strtotime($iterableNextChargeDate));
            }

            if (!empty($iterableEventName)) {

                $iterableEventName = strtolower(preg_replace('/\s+/', '', $iterableEventName));
                if (($iterableEventName == $eventName) && ($iterableSubscriptionId == $subscriptionId)) {

                    if ($iterableNextChargeDate > $currentDate) {

                        $eventCreateStatus = true;
                        break;
                    }

                }
            }
        }
        
        return $eventCreateStatus;
    }
}
