<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
//use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\Model\IterableApi;
use App\Helper\Helper;
use Log;

class IterableUserChannelUpdateEvent implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $helperObj;
    private $iterableObj;
    private $rechargeObj;
    private $customerEmail;
    //private $logFile;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($customerEmail)
    {
        $this->customerEmail = $customerEmail;
        //$this->logFile = $logPath;
        $this->iterableObj = new IterableApi();
        $this->helperObj = new Helper();
    }

    public function handle()
    {

        sleep(5);
        $customerDetails = $this->iterableObj->getIterableSubscription($this->customerEmail);

        if (isset($customerDetails->error) || isset($customerDetails->errors) || empty($customerDetails->user->email) ||
        empty($customerDetails)) {

            $status = "failed";
            $error = "Unknown error";
            if (isset($customerDetails->error)) {

                $error = $customerDetails->error;

            } else if (isset($customerDetails->errors)) {

                $error = "Request is not correct or missing the required parameters";

            }

            Log::channel('iterableWebhookLog')->info("IterableUserChannelUpdateEvent: Update iterable user channel status - 
            Failed to get the Iterable user details for CustomerEmail (".$this->customerEmail."), Error: " . $error);

            die();
        }

        $channelIds = env('ITERABLE_DEFAULT_OFF_CHANNEL_IDS',true);
        $channelIdExplode = explode(",",$channelIds);
        $updateRequest = array(
            "email" => $this->customerEmail,
            "unsubscribedChannelIds" => array_map('intval', $channelIdExplode)
        );

        $updateResponse = $this->iterableObj->updateUserIterableSubscription(json_encode($updateRequest));

        if(empty($updateResponse->code) || strtolower(trim($updateResponse->code)) == 'success') {

            Log::channel('iterableWebhookLog')->info("IterableUserChannelUpdateEvent: Update iterable user channel status - 
            Successfully Updated the channel status (".strtolower($customerDetails->user->email).")");
        } else {
            Log::channel('iterableWebhookLog')->info("IterableUserChannelUpdateEvent: Update iterable user channel status - 
            Failed to  Updated the channel status (".strtolower($customerDetails->user->email).")");
        }
    }
}
