<?php
namespace App\Model;
use Illuminate\Database\Eloquent\Model;

class IterableApi extends Model {

    private $iterableUrl;
    private $iterableApiKey;

    function __construct() {

        $this->iterableUrl = env('ITERABLE_URL', true);
        $this->iterableApiKey = env('ITERABLE_API_KEY',true);
    }

    /**
     * get subscription from Iterable
     */
    function getIterableSubscription($email = NULL) {

        $data = NULL;
        $method = 'GET';
        $path = $this->iterableUrl.'users/'.$email.'?api_key='.$this->iterableApiKey;
        $response = $this->call($method, $path, $data);
        return json_decode($response);

    }

    /**
     * get user Iterable subscription
     */
    function updateUserIterableSubscription($data = NULL) {

        $method = 'POST';
        $path = $this->iterableUrl.'users/updateSubscriptions?api_key='.$this->iterableApiKey;
        $response = $this->call($method, $path, $data);
        return json_decode($response);

    }

    /**
     * create new Iterable event
     */
    function createNewIterableEvent($data) {

        $method = 'POST';
        $path = $this->iterableUrl.'events/trackBulk?api_key='.$this->iterableApiKey;
        $response = $this->call($method, $path, $data);
        return json_decode($response);

    }

    /**
     * add subscriber to Iterable List
     */
    function addSubscriberToIterableList($data) {

        $method = 'POST';
        $path = $this->iterableUrl.'lists/subscribe?api_key='.$this->iterableApiKey;
        $response = $this->call($method, $path, $data);
        return json_decode($response);

    }

    /**
     * update user data fields
     */
    function updateUserDataFields($request) {

        $method = 'POST';
        $path = $this->iterableUrl.'users/update?api_key='.$this->iterableApiKey;
        $response = $this->call($method, $path, $request);
        return json_decode($response);

    }

    /**
     * Get events by Shopify customer Id
     */
    public function getIterableEventsByShopifyCustomerId($limit, $customerId) {

        $data = NULL;
        $method = 'GET';
        $path = $this->iterableUrl.'events/byUserId/'.$customerId.'?limit='.$limit.'&api_key='.$this->iterableApiKey;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * CURL call
     */
    function call($method, $path, $data = []) {

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING, '');
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);

        if ($method !== 'GET') {

            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }
     
        $response = curl_exec($curl);
        return $response;

    }
}
