<?php

use Monolog\Handler\StreamHandler;
use Monolog\Handler\SyslogUdpHandler;

return [

    /*
    |--------------------------------------------------------------------------
    | Default Log Channel
    |--------------------------------------------------------------------------
    |
    | This option defines the default log channel that gets used when writing
    | messages to the logs. The name specified in this option should match
    | one of the channels defined in the "channels" configuration array.
    |
    */

    'default' => env('LOG_CHANNEL', 'stack'),

    /*
    |--------------------------------------------------------------------------
    | Log Channels
    |--------------------------------------------------------------------------
    |
    | Here you may configure the log channels for your application. Out of
    | the box, Laravel uses the Monolog PHP logging library. This gives
    | you a variety of powerful log handlers / formatters to utilize.
    |
    | Available Drivers: "single", "daily", "slack", "syslog",
    |                    "errorlog", "monolog",
    |                    "custom", "stack"
    |
    */

    'channels' => [
        'stack' => [
            'driver' => 'stack',
            'channels' => ['daily'],
        ],

        'single' => [
            'driver' => 'single',
            'path' => storage_path('logs/lumen.log'),
            'level' => 'debug',
        ],

        'daily' => [
            'driver' => 'daily',
            'path' => storage_path('logs/lumen.log'),
            'level' => 'debug',
            'days' => 14,
        ],

        'abandonedCheckoutLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/abandonedCheckoutLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],

        'checkoutLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/checkoutLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],

        'giftRecipientCheckoutLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/giftRecipientCheckoutLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],

        'iterableLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/iterableLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],

        'iterableWebhookLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/iterableWebhookLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],

        'rechargeAddressLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/rechargeAddressLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],

        'subscriptionControllerLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/subscriptionControllerLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],
        'nextChargeDateChangeLog' => [
            'driver' => 'daily',
            'path' => storage_path('logs/nextChargeDateChangeLog.log'),
            'level' => 'debug',
            'days' => 14,
            'permission' => 0664,
        ],

        'slack' => [
            'driver' => 'slack',
            'url' => env('LOG_SLACK_WEBHOOK_URL'),
            'username' => 'Lumen Log',
            'emoji' => ':boom:',
            'level' => 'critical',
        ],

        'papertrail' => [
            'driver' => 'monolog',
            'level' => 'debug',
            'handler' => SyslogUdpHandler::class,
            'handler_with' => [
                'host' => env('PAPERTRAIL_URL'),
                'port' => env('PAPERTRAIL_PORT'),
            ],
        ],

        'stderr' => [
            'driver' => 'monolog',
            'handler' => StreamHandler::class,
            'with' => [
                'stream' => 'php://stderr',
            ],
        ],

        'syslog' => [
            'driver' => 'syslog',
            'level' => 'debug',
        ],

        'errorlog' => [
            'driver' => 'errorlog',
            'level' => 'debug',
        ],
    ],

];
